
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
#endregion

namespace SnakeGame
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public partial class Food : Microsoft.Xna.Framework.DrawableGameComponent
    {

        enum FoodState
        {
            invisible,
            visible
        }

        FoodState state;

        public Food(Game game)
            : base(game)
        {
        }

        private Texture2D segmentTexture;
        private Rectangle segmentRectangle;
        SpriteBatch spriteBatch;
        ContentManager content;
        SnakeGame parentGame;

        public int segmentsAcrossScreen = 20;
        public int segmentsUpScreen = 20;

        System.Random rand = new Random(1);
        int xLimit;
        int yLimit;

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            parentGame = this.Game as SnakeGame;
            content = parentGame.content;
            this.DrawOrder = 1;
            segmentRectangle = new Rectangle(
                0,
                0,
                this.Game.Window.ClientBounds.Width / segmentsAcrossScreen,
                this.Game.Window.ClientBounds.Height / segmentsUpScreen
                );

            xLimit = this.Game.Window.ClientBounds.Width - segmentRectangle.Width;
            yLimit = this.Game.Window.ClientBounds.Height - segmentRectangle.Height;

            state = FoodState.invisible;

            base.Initialize();
        }

        protected override void LoadGraphicsContent(bool loadAllContent)
        {
            if (loadAllContent)
            {
                segmentTexture = content.Load<Texture2D>(@"Content\Textures\cheese");
                spriteBatch = new SpriteBatch(this.GraphicsDevice);
            }
            base.LoadGraphicsContent(loadAllContent);
        }

        int foodLiklihoodPercent = 10;
        int minimumFoodLife = 20;
        int randomFoodLifeComponent = 100;
        int foodLife;

        public void PlaceFood()
        {
            if (state == FoodState.visible)
            {
                return;
            }

            segmentRectangle.X = rand.Next(xLimit);
            segmentRectangle.Y = rand.Next(yLimit);

            foodLife = minimumFoodLife + rand.Next(randomFoodLifeComponent);

            state = FoodState.visible;
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            switch (state)
            {
                case FoodState.invisible:
                    if (rand.Next(100) <= foodLiklihoodPercent)
                    {
                        PlaceFood(); 
                    }
                    break;
                case FoodState.visible:
                    if (--foodLife == 0)
                    {
                        state = FoodState.invisible;
                    }
                    if (parentGame.Player.HeadIntersectsWith(segmentRectangle))
                    {
                        state = FoodState.invisible;
                    }
                    break;
            }
            base.Update(gameTime);
        }

        public override void Draw(GameTime gameTime)
        {
            if (state == FoodState.invisible)
            {
                return;
            }

            spriteBatch.Begin(SpriteBlendMode.AlphaBlend);

            spriteBatch.Draw(segmentTexture, segmentRectangle, Color.White);

            spriteBatch.End();
            base.Draw(gameTime);
        }
    }
}


