
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
#endregion

namespace SnakeGame
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public partial class PlayerSnake : Microsoft.Xna.Framework.DrawableGameComponent
    {

        class SnakeBodySegment
        {

            public PlayerSnake snake;

            public Texture2D segmentTexture;

            public Rectangle segmentRectangle;

            public void Draw () 
            {
               snake.spriteBatch.Draw(segmentTexture, segmentRectangle, Color.White);
            }

            public SnakeBodySegment(
                PlayerSnake inPlayerSnake, 
                Texture2D inTexture, 
                Rectangle inRectangle)
            {
                snake = inPlayerSnake;
                segmentTexture = inTexture;
                segmentRectangle = inRectangle;
            }
        }

        public bool HeadIntersectsWith(Rectangle rect)
        {
            return !(snake[0].segmentRectangle.Left > rect.Right
                    || snake[0].segmentRectangle.Right < rect.Left
                    || snake[0].segmentRectangle.Top > rect.Bottom
                    || snake[0].segmentRectangle.Bottom < rect.Top
                    );
        }

        SnakeGame parent;

        public int segmentsAcrossScreen = 30;
        public int segmentsUpScreen = 30;
        public int maxSnakeLength = 100;

        SnakeBodySegment[] snake;

        public PlayerSnake(Game game)
            : base(game)
        {
            // TODO: Construct any child components here
        }

        private Texture2D segmentTexture;
        private Rectangle segmentRectangle;
        SpriteBatch spriteBatch;
        ContentManager content;

        int snakeLength = 1;

        int xSpeed = 0;
        int ySpeed = 0;

        int xMaxSpeed;
        int yMaxSpeed;

        int growCount = 0;
        int growLimit = 5;

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            parent = this.Game as SnakeGame;
            content = parent.content;
            this.DrawOrder = 1;
            segmentRectangle = new Rectangle(
                0, 
                0, 
                this.Game.Window.ClientBounds.Width / segmentsAcrossScreen, 
                this.Game.Window.ClientBounds.Height / segmentsUpScreen
                );
            xMaxSpeed = segmentRectangle.Width;
            yMaxSpeed = segmentRectangle.Height;
            snake = new SnakeBodySegment[maxSnakeLength];
            base.Initialize();
        }

        protected override void LoadGraphicsContent(bool loadAllContent)
        {
            if (loadAllContent)
            {
                segmentTexture = content.Load<Texture2D>(@"Content\Textures\SnakeSkin");
                spriteBatch = new SpriteBatch(this.GraphicsDevice);
            }

            for (int i = 0; i < maxSnakeLength; i++)
            {
                snake[i] = new SnakeBodySegment(this, segmentTexture, segmentRectangle);
            }

            base.LoadGraphicsContent(loadAllContent);
        }


        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {


            GamePadState padState = GamePad.GetState(PlayerIndex.One);

            if (padState.IsConnected)
            {

                if (padState.Buttons.Back == ButtonState.Pressed)
                {
                    parent.Exit();
                }

                if (padState.DPad.Down == ButtonState.Pressed)
                {
                    xSpeed = 0;
                    ySpeed = yMaxSpeed;
                }
                if (padState.DPad.Up == ButtonState.Pressed)
                {
                    xSpeed = 0;
                    ySpeed = -yMaxSpeed;
                }
                if (padState.DPad.Left == ButtonState.Pressed)
                {
                    xSpeed = -xMaxSpeed;
                    ySpeed = 0;
                }

                if (padState.DPad.Right == ButtonState.Pressed)
                {
                    xSpeed = xMaxSpeed;
                    ySpeed = 0;
                }
            }

            growCount = growCount + 1;

            if (growCount >= growLimit)
            {
                growCount = 0;
                if (snakeLength < (maxSnakeLength - 1))
                {
                    snakeLength = snakeLength + 1;
                }
            }

            for (int i = snakeLength; i > 0; i = i - 1)
            {
                snake[i].segmentRectangle.X = snake[i - 1].segmentRectangle.X;
                snake[i].segmentRectangle.Y = snake[i - 1].segmentRectangle.Y;
            }

            snake[0].segmentRectangle.X = snake[0].segmentRectangle.X + xSpeed;
            snake[0].segmentRectangle.Y = snake[0].segmentRectangle.Y + ySpeed;

            /// check for snake colliding with itself

            for (int i = 1; i < snakeLength; i = i + 1)
            {
                if (snake[0].segmentRectangle.X == snake[i].segmentRectangle.X
                    && snake[0].segmentRectangle.Y == snake[i].segmentRectangle.Y)
                {
                    //game.gameRunning = false;
                    break;
                }
            }

            base.Update(gameTime);
        }


        public override void Draw(GameTime gameTime)
        {
            spriteBatch.Begin(SpriteBlendMode.AlphaBlend);

            for (int i = 0; i < snakeLength; i = i + 1)
            {
                snake[i].Draw();
            }

            spriteBatch.End();
            base.Draw(gameTime);
        }
    }
}


